"use strict";
/*
* This program and the accompanying materials are made available under the terms of the
* Eclipse Public License v2.0 which accompanies this distribution, and is available at
* https://www.eclipse.org/legal/epl-v20.html
*
* SPDX-License-Identifier: EPL-2.0
*
* Copyright Contributors to the Zowe Project.
*
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModifyJobs = void 0;
const imperative_1 = require("@zowe/imperative");
const JobsConstants_1 = require("./JobsConstants");
const core_for_zowe_sdk_1 = require("@zowe/core-for-zowe-sdk");
/**
 * Class to handle modify of jobclass and holdStatus information
 * @export
 * @class ModifyJobs
 */
class ModifyJobs {
    /**
     * Modify a job
     * @static
     * @param {AbstractSession} session - z/OSMF connection info
     * @param {IModifyJobParms} parms - parms object (see IModifyJobParms interface for details)
     * @param {IModifyJobOptions} options - options object (see IModifyJobOptions interface for details)
     * @returns {Promise<undefined|IJobFeedback>} - promise of undefined, or IJobFeedback object returned by API if modifyVersion is 2.0
     * @memberof ModifyJobs
     */
    static modifyJob(session, parms, options) {
        return __awaiter(this, void 0, void 0, function* () {
            this.log.trace("ModifyJob called with jobname %s jobid %s", parms);
            return ModifyJobs.modifyJobCommon(session, parms, options);
        });
    }
    /**
     * Modify a job
     * Full version of the API with a parameter object
     * @static
     * @param {AbstractSession} session - z/OSMF connection info
     * @param {IModifyJobParms} parms - parm object (see IModifyJobParms interface for details)
     * @param {IModifyJobOptions} options - options object (see IModifyJobOptions interface for details)
     * @returns {Promise<undefined|IJobFeedback>} - promise of undefined, or IJobFeedback object returned by API if modifyVersion is 2.0
     * @memberof ModifyJobs
     */
    static modifyJobCommon(session, parms, options) {
        return __awaiter(this, void 0, void 0, function* () {
            this.log.trace("ModifyJobCommon called with parms %s", JSON.stringify(parms));
            this.log.info("Modifying job %s.%s", parms.jobname, parms.jobid);
            imperative_1.ImperativeExpect.keysToBeDefinedAndNonBlank(parms, ["jobid", "jobname"], "You must specify both the jobname and jobid for the job you want to modify.");
            const headers = [imperative_1.Headers.APPLICATION_JSON];
            const parameters = "/" + parms.jobname + "/" + parms.jobid;
            let response;
            let request;
            let mergedMessage = "";
            if (options.release || options.hold || options.jobclass) {
                if (options.hold || options.release) {
                    if (options.hold) {
                        request = { request: "hold" };
                    }
                    else {
                        request = { request: "release" };
                    }
                    try {
                        response = yield core_for_zowe_sdk_1.ZosmfRestClient.putExpectJSON(session, JobsConstants_1.JobsConstants.RESOURCE + parameters, headers, request);
                        mergedMessage = mergedMessage + '\n' + response.message;
                    }
                    catch (err) {
                        err.mMessage = err.mMessage.concat('Modification Error');
                        throw err;
                    }
                }
                // build request to change class, only if defined and no exception from potential previous request
                if (options.jobclass) {
                    request = {
                        class: options.jobclass,
                    };
                    response = yield core_for_zowe_sdk_1.ZosmfRestClient.putExpectJSON(session, JobsConstants_1.JobsConstants.RESOURCE + parameters, headers, request);
                    mergedMessage = mergedMessage + '\n' + response.message;
                }
                response.message = mergedMessage;
                return response;
            }
            else {
                throw new imperative_1.ImperativeError({ msg: "At least one option needs to be selected for modification: `hold`, `release`, `jobclass`" });
            }
        });
    }
    /**
     * @returns {Logger}
     */
    static get log() {
        return imperative_1.Logger.getAppLogger();
    }
}
exports.ModifyJobs = ModifyJobs;
//# sourceMappingURL=ModifyJobs.js.map